import argparse
import os
from .curses_app import download_app, browse_app
from .themes import get_themes_dir, set_theme, get_themedir


def scmd_init(args):
    themes_dir = None
    themes_dir = args.dir or os.path.join(os.path.dirname(__file__), "..", "themes")

    themes_dir = os.path.normpath(themes_dir)
    with open(os.path.normpath(os.path.join(os.path.dirname(__file__), "..", '.themedir')), 'w') as f:
        f.write(themes_dir)
    path = os.path.join(themes_dir, "__current.css")
    path = os.path.normcase(path)
    print(f"Initializing BeautifulDiscord at {path}")

    os.makedirs(themes_dir, exist_ok=True)

    os.system(f'beaudis --css "{path}"')


def scmd_browse(_):
    download_app()


def scmd_set(args):
    if args.file:
        print(f"Installing {args.file} theme.")
        set_theme(args.file)
    else:
        browse_app()


def scmd_clear(_):
    open(os.path.join(get_themes_dir(), "__current.css"), "w")


def scmd_revert(_):
    scmd_clear([])
    os.remove(get_themedir())
    os.system("beaudis --revert")


def scmd_cat(_):
    try:
        with open(os.path.join(get_themes_dir(), "__current.css"), 'r') as f:
            print(f.read())
    except FileNotFoundError:
        print('Execute "bdtheme init" first.')


def cmd_bdtheme():
    parser = argparse.ArgumentParser()
    subparsers = parser.add_subparsers(required=True)

    p_init = subparsers.add_parser(
        "init", help="Initialize BeautifulDiscord. Requires Discord running.")
    p_init.add_argument(
        "dir", metavar="themes_dir", nargs="?", default="", help="Custom themes location.")
    p_init.set_defaults(func=scmd_init)

    p_browse = subparsers.add_parser(
        "download", help="Download Discord themes. Requires internet connection.")
    p_browse.set_defaults(func=scmd_browse)

    p_set = subparsers.add_parser(
        "set", help="Choose a theme from installation folder.")
    p_set.add_argument("file", nargs="?", default=None,
                       help="Theme .css file location.")
    p_set.set_defaults(func=scmd_set)

    p_clear = subparsers.add_parser(
        "clear", help="Set Discord theme to default.")
    p_clear.set_defaults(func=scmd_clear)

    p_revert = subparsers.add_parser(
        "revert", help="Remove BeautifulDiscord .css hot-reload.")
    p_revert.set_defaults(func=scmd_revert)

    p_cat = subparsers.add_parser("cat", help="Print out current .css theme.")
    p_cat.set_defaults(func=scmd_cat)

    args = parser.parse_args()
    args.func(args)



import contextlib
import curses
import shutil
import re
import os
from .themes import parse_vsthemes_list, parse_better_list
from .themes import Paginator, save_theme, get_themes_dir, set_theme, get_themedir
from curses import wrapper


def init_app(stdscr, paginator_data, on_choice):
    curses.init_pair(1, curses.COLOR_RED,
                     curses.COLOR_WHITE)  # cursor position
    curses.init_pair(2, curses.COLOR_RED, curses.COLOR_BLACK)
    curses.curs_set(False)
    wx, wy = shutil.get_terminal_size()
    cursor = [5, 5]

    paginators = []
    theme_cache = []
    pag_index = 0
    if isinstance(paginator_data, dict):
        for k, v in paginator_data.items():
            p = Paginator(v, pagesize=wy - 5 - 5, name=k)
            paginators.append(p)
            theme_cache.append(p.next_page())
    elif isinstance(paginator_data, list):
        for i, v in enumerate(paginator_data):
            p = Paginator(v, pagesize=wy - 5 - 5, name=str(i + 1))
            paginators.append(p)
            theme_cache.append(p.next_page())
    else:
        p = Paginator(paginator_data, pagesize=wy - 5 - 5, name="main")
        paginators.append(p)
        theme_cache.append(p.next_page())

    mainloop = True
    while mainloop:
        paginator = paginators[pag_index]
        theme_list = theme_cache[pag_index]
        cursor[0] = min([max(5, cursor[0]), wy - 5, len(theme_list) + 4])

        stdscr.clear()
        # Slashes
        for y in range(wy):
            for x in range(wx):
                if (x == 0 or y == 0):
                    stdscr.addch(y, x, "#")

        # Notes text
        stdscr.addstr(3, 3, 'Press "q" to exit.')
        stdscr.addstr(wy - 3, 3, '"WaSd" or arrows to navigate.')
        stdscr.addstr(wy - 2, 3, 'Capital "AD" to switch tabs.')
        # No-init warning text
        if not os.path.exists(get_themedir()):
            stdscr.addstr(
                wy - 1, 10, 'Execute "bdtheme init" first!', curses.color_pair(2))
        # Themes text
        for y in range(min(wy - 5 - 5, len(theme_list))):
            try:
                stdscr.addstr(y + 5, 5, f"{str(y + 1 + paginator.pagesize * (paginator.page - 1))}. {str(theme_list[y])}")

            except IndexError:
                break

        # Current paginators
        tabs_text = [f"[ {p.name} ]" for p in paginators]
        overall_len = sum(len(i) for i in tabs_text)
        start_x = (wx // 2) - (overall_len // 2)
        offset = 0
        for tab in tabs_text:
            if tab == f"[ {paginator.name} ]":
                stdscr.addstr(2, start_x + offset, tab, curses.A_BOLD)
            else:
                stdscr.addstr(2, start_x + offset, tab)
            offset += len(tab)

        # Cursor position
        stdscr.addch(*cursor, stdscr.inch(*cursor), curses.color_pair(1))

        stdscr.refresh()

        key = stdscr.getch()
        if key in [ord("q"), ord("Q"), ord("й"), ord("Й")]:
            mainloop = False
        elif key in [curses.KEY_DOWN, ord("s"), ord("S"), ord("ы"), ord("Ы")]:
            # if cursor[0] + 1 in range(5, min(wy, len(theme_list)) + 5):
            cursor[0] += 1
        elif key in [curses.KEY_UP, ord("w"), ord("W"), ord("ц"), ord("Ц")]:
            # if cursor[0] - 1 in range(5, min(wy, len(theme_list)) + 5):
            cursor[0] -= 1
        elif key in [curses.KEY_ENTER, 10]:
            chosen_theme = theme_list[cursor[0] - 5]
            on_choice(chosen_theme)
            mainloop = False
        elif key in [curses.KEY_RIGHT, ord("d"), ord("в")]:
            if next_page := paginator.next_page():
                theme_cache[pag_index] = next_page
        elif key in [curses.KEY_LEFT, ord("a"), ord("ф")]:
            if prev_page := paginator.previous_page():
                theme_cache[pag_index] = prev_page
        elif key in [ord("A"), ord("Ф")]:
            pag_index -= 1
            if pag_index < 0:
                pag_index = len(paginators) - 1
        elif key in [ord("D"), ord("В")]:
            pag_index += 1
            if pag_index > (len(paginators) - 1):
                pag_index = 0


def download_app():
    def on_choice(theme):
        print(f"Downloading {theme.name}...")
        save_theme(theme)

    wrapper(init_app, {"betterdis": parse_better_list,
            "vsthemes": parse_vsthemes_list}, on_choice)


def browse_app():
    name_re = r'/\* <(.+)> @ (.+) \*/'

    files = []
    with contextlib.suppress(FileNotFoundError):
        files = os.listdir(get_themes_dir())

    all_themes_list = []
    vs_themes_list = []
    bd_themes_list = []
    themes_dict = {}
    for file in files:
        filename = os.path.join(get_themes_dir(), file)
        if file == "__current.css":
            continue
        with open(filename, 'r', encoding="utf-8") as f:
            match = re.match(name_re, f.read().split('\n')[0])
            theme_name, theme_src = match.group(1), match.group(2)

            all_themes_list.append(theme_name)
            if theme_src == "vs":
                vs_themes_list.append(theme_name)
            elif theme_src == "bd":
                bd_themes_list.append(theme_name)
            themes_dict[theme_name] = filename

    def on_choice(theme_name):
        print(f"Setting {theme_name}...")
        filename = themes_dict[theme_name]
        set_theme(filename)

    def retrieve_all(page):
        return all_themes_list if page == 1 else []

    def retrieve_bd(page):
        return bd_themes_list if page == 1 else []

    def retrieve_vs(page):
        return vs_themes_list if page == 1 else []

    wrapper(init_app, {"overall": retrieve_all,
            "betterdis": retrieve_bd, "vsthemes": retrieve_vs}, on_choice)



from setuptools import setup, find_packages
import sys

requirements = ["bs4", "requests", "lxml",
                "beaudis"]
if sys.platform == "win32":
    requirements.append("windows-curses")

with open("README.md", "r") as readme:
    setup(
        name="bdtheme",
        author="TheAmmiR",
        license="MIT",
        description="Console BeautifulDiscord theme manager",
        long_description=readme.read(),
        long_description_content_type='text/markdown',
        version="1.3.22",
        packages=find_packages(
            exclude=["themes", "venv"]),
        install_requires=requirements,
        python_requires=">=3.8",
        entry_points={
            "console_scripts": [
                "bdtheme=bdtheme.src.main:cmd_bdtheme"
            ]
        }
    )



import requests as rq
import os
import shutil
import uuid
from typing import Tuple, List, Callable
from bs4 import BeautifulSoup as BS

USER_AGENT = {
    "User-Agent": "Mozilla/5.0 (Windows NT 10.0; Win64; x64; rv:95.0) Gecko/20100101 Firefox/95.0"}


class Theme:
    """
        :param img_url: str - theme screenshot url
        :param url: str - theme page url
        :param likes: int - likes count
        :param views: int - views count
        :param css: Callable - function to parse css
        :param source: str{bd, vs} - website the theme is from
    """

    def __init__(self, name: str, **kwargs):
        self.name = name
        self.img_url = kwargs.get("img_url")
        self.url = kwargs.get("url")
        self.likes = kwargs.get("likes")
        self.views = kwargs.get("views")
        self.css = kwargs.get("css")
        self.source = kwargs.get("source")

    def __str__(self):
        return self.name


class Paginator:
    def __init__(self, retrieve_new: Callable, *, pagesize: int, name: str):
        self.retrieve_new = retrieve_new
        self.pagesize = pagesize

        self.page = 0
        self.parsepage = 1
        self._coll = self.retrieve_new(self.parsepage)

        self.pages_cache = {}

        self.name = name or "paginator"

    def next_page(self):
        if (cache := self.pages_cache.get(self.page + 1)):
            self.page += 1
            return cache

        result = []
        remaining = self.pagesize

        while remaining != 0:
            page_part = self._coll[:remaining]
            result += page_part
            self._coll = self._coll[remaining:]
            remaining -= len(page_part)

            if not self._coll:
                self.parsepage += 1
                self._coll = self.retrieve_new(self.parsepage)
            if not self._coll:
                if result:
                    break
                return result

        if result:
            self.pages_cache[self.page + 1] = result
            self.page += 1
        return result

    def previous_page(self):
        if (cache := self.pages_cache.get(self.page - 1)):
            self.page -= 1
            return cache


def get_themedir():
    return os.path.normpath(os.path.join(
        os.path.dirname(__file__), "..", ".themedir"))


def get_themes_dir():
    themedir_path = get_themedir()
    if os.path.exists(themedir_path):
        with open(themedir_path, "r", encoding="utf-8") as f:
            return f.read().strip()
    else:
        return os.path.normpath(os.path.join(
            os.path.dirname(__file__), "..", "themes"))


def set_theme(file: str):
    shutil.copyfile(file, os.path.join(get_themes_dir(), "__current.css"))


def parse_vsthemes_list(page: int = 1) -> List[Theme]:
    base_url = 'https://vsthemes.org/en/skins/discord/page/{page}/'

    resp = rq.get(base_url.format(page=page), headers=USER_AGENT)
    if not resp.ok:
        return []

    bs = BS(resp.text, 'lxml')

    theme_elements = bs.find_all("div", {"class": "infiniteBlock"})

    result = []
    for el in theme_elements:
        try:
            name = el.find_all("div")[2].find("a").text
        except IndexError:
            continue
        img_url = "https://vsthemes.org" + el.find("img").get("src")
        theme_url = el.find("a").get("href")
        views, _, likes = el.find("ul", {"class": "iOptions"}).find_all("span")
        views = int(views.text.replace(" ", ""))
        likes = int(likes.text.replace(" ", ""))

        result.append(Theme(name, img_url=img_url,
                      url=theme_url, likes=likes, views=views, css=parse_vsthemes_css, source="vs"))

    return result


def parse_vsthemes_css(url: str) -> Tuple[str, str]:
    resp = rq.get(url, headers=USER_AGENT)
    bs = BS(resp.text, 'lxml')

    css_text = bs.find("code", {"class": "php"})

    return css_text.text


def parse_better_list(page: int, pages: int = 10):
    abbrevs = {"k": 1000, "m": 1000000}
    base_url = "https://betterdiscord.app/Addon/GetApprovedAddons?type=theme&filter=&page={page}&pages={pages}&sort=popular&sortDirection=&tags=[]"

    resp = rq.get(base_url.format(page=page,
                  pages=pages), headers=USER_AGENT)
    if not resp.ok:
        return []

    bs = BS(resp.text, 'lxml')

    card_list = bs.find_all("a", {"class": "card-wrap"})
    result = []
    for el in card_list:
        name = el.find("h3", {"class": "card-title"}).text
        img_url = "https://betterdiscord.app" + el.find("img").get("src")
        theme_url = "https://betterdiscord.app" + el.get("href")
        views = el.find("div", {"id": "addon-downloads"}
                        ).text.strip().replace(",", '')
        likes = el.find("div", {"id": "addon-likes"}
                        ).text.strip().replace(",", "")

        views = int(views) if views.isdigit() else int(float(views[:-1]) * abbrevs[views[-1].lower()])

        likes = int(likes) if likes.isdigit() else int(float(likes[:-1]) * abbrevs[likes[-1].lower()])


        result.append(Theme(name, img_url=img_url,
                      url=theme_url, likes=likes, views=views, css=parse_better_css, source="bd"))

    return result


def parse_better_css(url: str) -> None:
    theme_resp = rq.get(url, headers=USER_AGENT)
    bs = BS(theme_resp.text, 'lxml')

    download_link = "https://betterdiscord.app" + \
        bs.find("a", {"class": "btn-primary"}).get("href")
    download_resp = rq.get(
        download_link, headers=USER_AGENT, allow_redirects=True)

    return download_resp.text


def save_theme(theme: Theme) -> None:
    themes_dir = get_themes_dir()
    theme_css = theme.css(theme.url)
    # with open(os.path.join(os.path.dirname(__file__), "fix.css"), "r") as f:
    #     fix_css = f.read()
    with open(os.path.join(themes_dir, f"{uuid.uuid4()}.css"), "a+", encoding="utf-8") as f:
        f.write(f"/* <{theme.name}> @ {theme.source} */\n")
        # f.write(fix_css)
        f.write("\n".join(list(filter(lambda i: not i.lower(
        ).startswith("//meta"), theme_css.split("\n")))))
